# Generated by Django 3.1.14 on 2022-02-02 07:12

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('topology', '0013_add_user_defined_properties_field'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='snapshot',
            name='organization',
        ),
    ]
