input[type='text'].readonly {
    border: 1px solid rgba(0, 0, 0, 0.05) !important;
    background-color: rgba(0, 0, 0, 0.07);
}

#id_receive_url {
    width: 435px;
}

.djnjg-overlay {
    display: none;
    position: fixed;
    left: 0;
    top: 0;
    z-index: 10000;
    width: 100%;
    height: 100%;
    background: rgba(0, 0, 0, 0.95);
}

.djnjg-overlay .inner {
    width: 100%;
    height: 100%;
    overflow: hidden;
}

.submit-row .visualizelink {
    background: #778898;
}
.submit-row .visualizelink:hover,
.submit-row .visualizelink:focus {
    background: #576a7c;
}

/*
The CSS for "hidden" class in Django >= 4.1 sets
"display: none !important". This hides the
"sideBarHandle" button. Thus, we unset the display
property below.
*/
.njg-sideBar.hidden {
    display: unset !important;
}
